@@ -17,7 +17,7 @@ class TemplateTileLayout(RETileLayout):
             patterns.append(prematch_pattern)
             patterns.append(r'\d+')
             filename_patterns.append(prematch_pattern)
-            filename_patterns.append(r'(?P<%s>\d+)' % match.group(1))
+            filename_patterns.append(r'(?P<{0!s}>\d+)'.format(match.group(1)))
             index = match.end()
         postmatch_pattern = re.escape(self.template[index:])
         patterns.append(postmatch_pattern)
@@ -28,7 +28,8 @@ class TemplateTileLayout(RETileLayout):
 
     def filename(self, tilecoord):
         return self.template % \
-                dict(z=tilecoord.z, x=tilecoord.x, y=tilecoord.y)
+            dict(z=tilecoord.z, x=tilecoord.x, y=tilecoord.y)
 
-    def _tilecoord(self, match):
+    @staticmethod
+    def _tilecoord(match):
         return TileCoord(*(int(match.group(s)) for s in 'zxy'))
