@@ -1,8 +1,18 @@
-from cStringIO import StringIO
-
-from PIL import Image
+from sys import version_info
 
 from tilecloud import Tile, TileStore
+from tilecloud.lib.PIL_ import FORMAT_BY_CONTENT_TYPE
+
+if version_info[0] == 2:
+    from cStringIO import StringIO
+else:
+    from io import BytesIO as StringIO
+
+try:
+    from PIL import Image
+    Image  # suppress pyflakes warning
+except:
+    import Image
 
 
 class MetaTileSplitterTileStore(TileStore):
@@ -21,5 +31,5 @@ class MetaTileSplitterTileStore(TileStore):
                 y = self.border + (tilecoord.y - metatile.tilecoord.y) * self.tile_size
                 image = metaimage.crop((x, y, x + self.tile_size, y + self.tile_size))
                 string_io = StringIO()
-                image.save(string_io, self.format)
-                yield Tile(tilecoord, data=string_io.getvalue())
+                image.save(string_io, FORMAT_BY_CONTENT_TYPE[self.format])
+                yield Tile(tilecoord, data=string_io.getvalue(), content_type=self.format)
